"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var session_exports = {};
__export(session_exports, {
  PreparedQuery: () => PreparedQuery,
  SQLJsSession: () => SQLJsSession,
  SQLJsTransaction: () => SQLJsTransaction
});
module.exports = __toCommonJS(session_exports);
var import_entity = require("../entity.cjs");
var import_logger = require("../logger.cjs");
var import_sql = require("../sql/index.cjs");
var import_sqlite_core = require("../sqlite-core/index.cjs");
var import_session = require("../sqlite-core/session.cjs");
var import_utils = require("../utils.cjs");
class SQLJsSession extends import_session.SQLiteSession {
  constructor(client, dialect, schema, options = {}) {
    super(dialect);
    this.client = client;
    this.schema = schema;
    this.logger = options.logger ?? new import_logger.NoopLogger();
  }
  static [import_entity.entityKind] = "SQLJsSession";
  logger;
  prepareQuery(query, fields, executeMethod) {
    const stmt = this.client.prepare(query.sql);
    return new PreparedQuery(stmt, query, this.logger, fields, executeMethod);
  }
  prepareOneTimeQuery(query, fields, executeMethod, customResultMapper) {
    const stmt = this.client.prepare(query.sql);
    return new PreparedQuery(
      stmt,
      query,
      this.logger,
      fields,
      executeMethod,
      customResultMapper,
      true
    );
  }
  transaction(transaction, config = {}) {
    const tx = new SQLJsTransaction("sync", this.dialect, this, this.schema);
    this.run(import_sql.sql.raw(`begin${config.behavior ? ` ${config.behavior}` : ""}`));
    try {
      const result = transaction(tx);
      this.run(import_sql.sql`commit`);
      return result;
    } catch (err) {
      this.run(import_sql.sql`rollback`);
      throw err;
    }
  }
}
class SQLJsTransaction extends import_sqlite_core.SQLiteTransaction {
  static [import_entity.entityKind] = "SQLJsTransaction";
  transaction(transaction) {
    const savepointName = `sp${this.nestedIndex + 1}`;
    const tx = new SQLJsTransaction("sync", this.dialect, this.session, this.schema, this.nestedIndex + 1);
    tx.run(import_sql.sql.raw(`savepoint ${savepointName}`));
    try {
      const result = transaction(tx);
      tx.run(import_sql.sql.raw(`release savepoint ${savepointName}`));
      return result;
    } catch (err) {
      tx.run(import_sql.sql.raw(`rollback to savepoint ${savepointName}`));
      throw err;
    }
  }
}
class PreparedQuery extends import_session.SQLitePreparedQuery {
  constructor(stmt, query, logger, fields, executeMethod, customResultMapper, isOneTimeQuery = false) {
    super("sync", executeMethod, query);
    this.stmt = stmt;
    this.logger = logger;
    this.fields = fields;
    this.customResultMapper = customResultMapper;
    this.isOneTimeQuery = isOneTimeQuery;
  }
  static [import_entity.entityKind] = "SQLJsPreparedQuery";
  run(placeholderValues) {
    const params = (0, import_sql.fillPlaceholders)(this.query.params, placeholderValues ?? {});
    this.logger.logQuery(this.query.sql, params);
    const result = this.stmt.run(params);
    if (this.isOneTimeQuery) {
      this.free();
    }
    return result;
  }
  all(placeholderValues) {
    const { fields, joinsNotNullableMap, logger, query, stmt, isOneTimeQuery, customResultMapper } = this;
    if (!fields && !customResultMapper) {
      const params = (0, import_sql.fillPlaceholders)(query.params, placeholderValues ?? {});
      logger.logQuery(query.sql, params);
      stmt.bind(params);
      const rows2 = [];
      while (stmt.step()) {
        rows2.push(stmt.getAsObject());
      }
      if (isOneTimeQuery) {
        this.free();
      }
      return rows2;
    }
    const rows = this.values(placeholderValues);
    if (customResultMapper) {
      return customResultMapper(rows, normalizeFieldValue);
    }
    return rows.map((row) => (0, import_utils.mapResultRow)(fields, row.map((v) => normalizeFieldValue(v)), joinsNotNullableMap));
  }
  get(placeholderValues) {
    const params = (0, import_sql.fillPlaceholders)(this.query.params, placeholderValues ?? {});
    this.logger.logQuery(this.query.sql, params);
    const { fields, stmt, isOneTimeQuery, joinsNotNullableMap, customResultMapper } = this;
    if (!fields && !customResultMapper) {
      const result = stmt.getAsObject(params);
      if (isOneTimeQuery) {
        this.free();
      }
      return result;
    }
    const row = stmt.get(params);
    if (isOneTimeQuery) {
      this.free();
    }
    if (!row || row.length === 0 && fields.length > 0) {
      return void 0;
    }
    if (customResultMapper) {
      return customResultMapper([row], normalizeFieldValue);
    }
    return (0, import_utils.mapResultRow)(fields, row.map((v) => normalizeFieldValue(v)), joinsNotNullableMap);
  }
  values(placeholderValues) {
    const params = (0, import_sql.fillPlaceholders)(this.query.params, placeholderValues ?? {});
    this.logger.logQuery(this.query.sql, params);
    this.stmt.bind(params);
    const rows = [];
    while (this.stmt.step()) {
      rows.push(this.stmt.get());
    }
    if (this.isOneTimeQuery) {
      this.free();
    }
    return rows;
  }
  free() {
    return this.stmt.free();
  }
}
function normalizeFieldValue(value) {
  if (value instanceof Uint8Array) {
    if (typeof Buffer !== "undefined") {
      if (!(value instanceof Buffer)) {
        return Buffer.from(value);
      }
      return value;
    }
    if (typeof TextDecoder !== "undefined") {
      return new TextDecoder().decode(value);
    }
    throw new Error("TextDecoder is not available. Please provide either Buffer or TextDecoder polyfill.");
  }
  return value;
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  PreparedQuery,
  SQLJsSession,
  SQLJsTransaction
});
//# sourceMappingURL=session.cjs.map