"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var session_exports = {};
__export(session_exports, {
  PreparedQuery: () => PreparedQuery,
  SQLiteProxyTransaction: () => SQLiteProxyTransaction,
  SQLiteRemoteSession: () => SQLiteRemoteSession
});
module.exports = __toCommonJS(session_exports);
var import_entity = require("../entity.cjs");
var import_logger = require("../logger.cjs");
var import_sql = require("../sql/index.cjs");
var import_sqlite_core = require("../sqlite-core/index.cjs");
var import_session = require("../sqlite-core/session.cjs");
var import_utils = require("../utils.cjs");
class SQLiteRemoteSession extends import_session.SQLiteSession {
  constructor(client, dialect, schema, options = {}) {
    super(dialect);
    this.client = client;
    this.schema = schema;
    this.logger = options.logger ?? new import_logger.NoopLogger();
  }
  static [import_entity.entityKind] = "SQLiteRemoteSession";
  logger;
  prepareQuery(query, fields, executeMethod) {
    return new PreparedQuery(this.client, query, this.logger, fields, executeMethod);
  }
  async transaction(transaction, config) {
    const tx = new SQLiteProxyTransaction("async", this.dialect, this, this.schema);
    await this.run(import_sql.sql.raw(`begin${config?.behavior ? " " + config.behavior : ""}`));
    try {
      const result = await transaction(tx);
      await this.run(import_sql.sql`commit`);
      return result;
    } catch (err) {
      await this.run(import_sql.sql`rollback`);
      throw err;
    }
  }
}
class SQLiteProxyTransaction extends import_sqlite_core.SQLiteTransaction {
  static [import_entity.entityKind] = "SQLiteProxyTransaction";
  async transaction(transaction) {
    const savepointName = `sp${this.nestedIndex}`;
    const tx = new SQLiteProxyTransaction("async", this.dialect, this.session, this.schema, this.nestedIndex + 1);
    await this.session.run(import_sql.sql.raw(`savepoint ${savepointName}`));
    try {
      const result = await transaction(tx);
      await this.session.run(import_sql.sql.raw(`release savepoint ${savepointName}`));
      return result;
    } catch (err) {
      await this.session.run(import_sql.sql.raw(`rollback to savepoint ${savepointName}`));
      throw err;
    }
  }
}
class PreparedQuery extends import_session.SQLitePreparedQuery {
  constructor(client, query, logger, fields, executeMethod) {
    super("async", executeMethod, query);
    this.client = client;
    this.logger = logger;
    this.fields = fields;
  }
  static [import_entity.entityKind] = "SQLiteProxyPreparedQuery";
  run(placeholderValues) {
    const params = (0, import_sql.fillPlaceholders)(this.query.params, placeholderValues ?? {});
    this.logger.logQuery(this.query.sql, params);
    return this.client(this.query.sql, params, "run");
  }
  async all(placeholderValues) {
    const { fields, query, logger, joinsNotNullableMap } = this;
    const params = (0, import_sql.fillPlaceholders)(query.params, placeholderValues ?? {});
    logger.logQuery(query.sql, params);
    const { rows } = await this.client(query.sql, params, "all");
    if (fields) {
      return rows.map((row) => (0, import_utils.mapResultRow)(fields, row, joinsNotNullableMap));
    }
    return rows;
  }
  async get(placeholderValues) {
    const { fields, query, logger, joinsNotNullableMap } = this;
    const params = (0, import_sql.fillPlaceholders)(query.params, placeholderValues ?? {});
    logger.logQuery(query.sql, params);
    const clientResult = await this.client(query.sql, params, "get");
    if (fields) {
      if (clientResult.rows === void 0) {
        return void 0;
      }
      return (0, import_utils.mapResultRow)(fields, clientResult.rows, joinsNotNullableMap);
    }
    return clientResult.rows;
  }
  async values(placeholderValues) {
    const params = (0, import_sql.fillPlaceholders)(this.query.params, placeholderValues ?? {});
    this.logger.logQuery(this.query.sql, params);
    const clientResult = await this.client(this.query.sql, params, "values");
    return clientResult.rows;
  }
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  PreparedQuery,
  SQLiteProxyTransaction,
  SQLiteRemoteSession
});
//# sourceMappingURL=session.cjs.map